//!
//!

use super::models;
use crate::repositories;
use diesel::{result::Error, sql_query, RunQueryDsl};
use std::fmt::Write;

static MEMBER_QUERY: &str = "
SELECT
  mbw.id                     as mbw_id,
  mbw.member_id              as mbw_member_id,
  mbw.bank_name              as mbw_bank_name,
  mbw.name                   as mbw_name,
  mbw.account_number         as mbw_account_number,
  mbw.amount                 as mbw_amount,
  mbw.password               as mbw_password,
  mbw.memo                   as mbw_memo,
  mbw.state                  as mbw_state,
  mbw.state_changed_at       as mbw_state_changed_at,
  mbw.created_at             as mbw_created_at,
  mbw.updated_at             as mbw_updated_at,
  
  _m.id                     as _m_id,
  _m.site_id                as _m_site_id,
  _m.member_class_id        as _m_member_class_id,  
  _m.member_level_id        as _m_member_level_id,    
  _m.username               as _m_username,  
  _m.password               as _m_password,  
  _m.nickname               as _m_nickname,
  _m.mobile_phone_number    as _m_mobile_phone_number,
  _m.state                  as _m_state,
  _m.state_changed_at       as _m_state_changed_at,
  _m.parent_member_id       as _m_parent_member_id,
  _m.child_member_count     as _m_child_member_count,
  _m.last_signined_ip       as _m_last_signined_ip,
  _m.last_signined_at       as _m_last_signined_at,
  _m.created_at             as _m_created_at,
  _m.updated_at             as _m_updated_at,
  _m.deleted_at             as _m_deleted_at

FROM member_bank_withdraws as mbw
  INNER JOIN members _m
    ON _m.id = mbw.member_id
";

pub struct Composition {}

impl std::fmt::Debug for Composition {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Composition of members").finish()
  }
}

impl Default for Composition {
  fn default() -> Self {
    Self::new()
  }
}

impl Composition {
  ///
  pub fn new() -> Composition {
    Composition {}
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::MemberBankWithdrawModel>, Error> {
    let query = format!(
      "
    {}
    WHERE
      mbw.id = $1
    ",
      MEMBER_QUERY
    );

    match sql_query(query)
      .bind::<diesel::sql_types::Uuid, _>(id)
      .get_result::<models::MemberBankWithdrawModel>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn selects(
    &self,
    conn: &diesel::PgConnection,
    ids: Vec<uuid::Uuid>,
  ) -> Result<Vec<models::MemberBankWithdrawModel>, Error> {
    if ids.is_empty() {
      return Ok(vec![]);
    }

    let mut query_where_id_in = String::new();

    for id in ids {
      if !query_where_id_in.is_empty() {
        write!(&mut query_where_id_in, " , ")
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      write!(&mut query_where_id_in, " '{}' ", id)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    let query = format!(
      "
    {}
    WHERE
      mbw.id IN ({})
    ",
      MEMBER_QUERY, query_where_id_in
    );

    println!("query: {}", query);

    match sql_query(query).get_results::<models::MemberBankWithdrawModel>(conn) {
      Ok(m) => Ok(m),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(vec![]),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_by_username(
    &self,
    conn: &diesel::PgConnection,
    username: &str,
  ) -> Result<Option<models::MemberBankWithdrawModel>, Error> {
    let query = format!(
      "
      {}
      WHERE
        _m.username = $1
      ",
      MEMBER_QUERY
    );

    match sql_query(query)
      .bind::<diesel::sql_types::Text, _>(username)
      .get_result::<models::MemberBankWithdrawModel>(conn)
    {
      Ok(m) => Ok(Some(m)),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(None),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &repositories::member_bank_withdraw::models::FindAll,
  ) -> Result<Vec<models::MemberBankWithdrawModel>, Error> {
    let mut query = String::new();
    write!(&mut query, "{}", MEMBER_QUERY)
      .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;

    let mut query_where = String::new();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.member_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mbw.member_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.bank_name_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mbw.bank_name like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.name_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mbw.name like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.account_number_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mbw.account_number like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = s.amount {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mbw.amount = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.memo_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mbw.memo like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = s.state {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "mbw.state = '{:?}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
    }

    if !query_where.is_empty() {
      write!(&mut query, " WHERE {}", query_where)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    println!("query: {}", query);

    match sql_query(query).get_results::<models::MemberBankWithdrawModel>(conn) {
      Ok(m) => Ok(m),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(vec![]),
        _ => Err(e),
      },
    }
  }
}
