use super::schema::{member_bank_deposits, MemberBankDepositState};
use beteran_common_rust as bcr;

///
#[derive(Identifiable, Queryable, PartialEq, Debug, Clone)]
#[table_name = "member_bank_deposits"]
pub struct MemberBankDeposit {
  ///
  pub id: uuid::Uuid,
  ///
  pub member_id: uuid::Uuid,
  ///
  pub name: String,
  ///
  pub amount: f64,
  ///
  pub memo: Option<String>,
  ///
  pub state: MemberBankDepositState,
  ///
  pub state_changed_at: i64,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
}

///
#[derive(Insertable, Debug, Clone)]
#[table_name = "member_bank_deposits"]
pub struct NewMemberBankDeposit {
  ///
  pub member_id: uuid::Uuid,
  ///
  pub name: String,
  ///
  pub amount: f64,
  ///
  pub memo: Option<String>,
}

///
#[derive(AsChangeset, Debug, Clone)]
#[table_name = "member_bank_deposits"]
pub struct ModifyMemberBankDeposit {
  ///
  pub name: String,
  ///
  pub amount: f64,
  ///
  pub memo: Option<String>,
}

///
#[derive(AsChangeset, Debug, Clone)]
#[table_name = "member_bank_deposits"]
pub struct ModifyMemberBankDepositForState {
  ///
  pub state: MemberBankDepositState,
}

///
#[derive(Debug, Clone)]
pub struct FindAllSearch {
  ///
  pub member_id: Option<uuid::Uuid>,
  ///
  pub name_like: Option<String>,
  ///
  pub amount: Option<f64>,
  ///
  pub memo_like: Option<String>,
  ///
  pub state: Option<MemberBankDepositState>,
}

///
#[derive(Debug, Clone)]
pub struct FindAll {
  ///
  pub search: Option<FindAllSearch>,
  ///
  pub pagination: Option<bcr::pagination::Pagination>,
  ///
  pub sorts: Option<Vec<bcr::pagination::Sort>>,
}
